#include "vect3.h"
#include <assert.h>
#include "picture.h"
#include "raycasting.h"

void unit_test_dot_cross() {
    Vect3 a(1, 4, 7);
    Vect3 b(9, 6, 5);
    Vect3 c = cross(a, b);
    Vect3 d = cross(b, a);

    assert(c == -d);
    assert(dot(a,c)==0);
    assert(dot(b,c)==0);
}

void unit_test_angles() {
    Vect3 a(1, 4, 7);
    Vect3 b(9, 6, 5);

    Vect3 c = cross(a, b);

    assert(are_equal(degrees_between(a, c), 90));
}

int main(){
    Vect3 p = Vect3(3, 5, 6);
    Vect3 q = Vect3();
    (p+q).print_coordinates();
    
    p.scale(5);
    p.print_coordinates();

    p.scaled(5).print_coordinates();

    Vect3 position(1,2,3), direction(1,2,3);
    position = position.added(direction.scaled(5));
    position.print_coordinates();
    position = position + direction*5;
    position.print_coordinates();
    unit_test_dot_cross();
    unit_test_angles();

    Picture pic = Picture(10, 10);
    pic.fill_whit_checkerboard();
    pic.print_to_screen();
    pic.save_as_pnm("image.pnm");

    Scene scene;
    scene.populate();
    Picture pic2 = render(1000, scene);
    pic2.print_to_screen();
    pic2.save_as_pnm("image2.pnm");
}