#include <iostream>
#include <fstream>
#include "picture.h"

Picture::Picture(int x, int y): res_x(x), res_y(y) {

    pixels.resize(x*y);
}

char to_char (Pixel x){
    Scalar grey_scale = x.length();
    switch (int(grey_scale*10)) {
        case 0: return ' ';
        case 1: return '.';
        case 2: return ',';  
        case 3: return ':';
        case 4: return ';';
        case 5: return '/';
        case 6: return '|';
        case 7: return '#';
        case 8: return '#';
        case 9: return '#';
        default: return '#';
    }
}

void Picture::fill_whit_checkerboard() {
    Pixel colA (0, 0, 0);
    Pixel colB (0, 1, 0);

    for (int y=0; y < res_y; y++)
    for (int x=0; x < res_x; x++)
        pixel(x, y) = ((x+y)%2) ? colA: colB;
}

void Picture::print_to_screen() const {
    for (int y=0; y < res_y; y++){
        for (int x=0; x < res_x; x++){
            std::cout<< to_char(pixel(x, y)) << ' ';
        }
        std::cout<< std::endl;
    }
}

void Picture::save_as_pnm(const std::string &filename) const{
    std::ofstream file (filename);
    file << "P3" << std::endl << res_x << ' ' << res_y << std::endl << 255 << std::endl;
    for (int y=0; y < res_y; y++){
        for (int x=0; x < res_x; x++) {
            Pixel p = pixel(x, y);
            file << int(clamp(p.x, 0, 1)*255) << ' ';
            file << int(clamp(p.y, 0, 1)*255) << ' ';
            file << int(clamp(p.z, 0, 1)*255) << ' ';        
        }
        file<< std::endl;
    }
    file.close();
}

Pixel& Picture::pixel(int x, int y) {
        return pixels[y*res_x + x];
}

Pixel Picture::pixel(int x, int y) const {
        return pixels[y*res_x + x];
}