#include "raycasting.h"


Picture render(int res, Scene &scene){

    Picture p(res, res);
    const Scalar focal_length = 1;
    const Vect3 POV(0, 1, -3);
    const Vect3 background(0, 0.5, 1);

    for (int y=0; y < p.res_y; y++){
        for (int x=0; x < p.res_x; x++) {
            Vect3 q(
                (x - p.res_x/2.)/(p.res_x/2.),
                -(y - p.res_y/2.)/(p.res_y/2.),
                focal_length
            );
            Ray r(POV, q);
            Scalar k_min = MISS;
            int hit_sphere = -1;

            for (int i = 0; i < scene.spheres.size(); i++){
                Sphere s = scene.spheres.at(i);
                Scalar k = r.intersect(s);
                if(k < k_min){
                    k_min = k;
                    hit_sphere = i;
                }
            }

            if(k_min == MISS){
                p.pixel(x, y) = background;
            }else{
                Vect3 hit_pos = r.pos(k_min);
                Vect3 hit_normal = scene.spheres.at(hit_sphere).normal_of(hit_pos);
                Vect3 view_dir = -r.direction;
                p.pixel(x, y) = scene.lighting(
                    view_dir,
                    hit_normal,
                    hit_pos
                );
            }
        }
    }

    return p;
}