#pragma once

#include <iostream>
#include "scalar.h"

struct Vect3;

Scalar dot(const Vect3 &a, const Vect3 &b);

Vect3 cross(const Vect3 &a, const Vect3 &b);

Scalar degrees_between(const Vect3 &a, const Vect3 &b);

struct Vect3 {
    Scalar x, y, z;

    static const int dimensionality;

    Vect3(Scalar _x, Scalar _y, Scalar _z): x(_x), y(_y), z(_z) {
        //this->x = _x; this->y = _y; this->z = _z;
    }

    //Vect3(): Vect3(0,0,0) {}

    Vect3(): x(0), y(0), z(0) {}

    void scale (Scalar c) {
        x *= c;
        y *= c;
        z *= c;
    }

    Vect3 scaled (Scalar k) const {
        return Vect3(x*k, y*k, z*k);
    }

    void add(const Vect3 &o) {
        x += o.x;
        y += o.y;
        z += o.z;
    }

    Vect3 added (const Vect3 &q) const {
        return Vect3(x+q.x, y+q.y, z+q.z);
    }

    Scalar length () const {
        return std::sqrt(dot(*this, *this));
    }

    void normalize () {
        scale(1/length());
    }

    Vect3 normalized () {
        return scaled(1/length());
    }

    Vect3 operator + (const Vect3 &q) const {
        return Vect3(x+q.x, y+q.y, z+q.z);
    }

    void operator +=(const Vect3 &o) {
        x += o.x;
        y += o.y;
        z += o.z;
    }

    Vect3 operator - () const {
        return Vect3(-x, -y, -z);
    }

    Vect3 operator -(const Vect3 &o) const {
        return Vect3(x - o.x, y - o.y, z - o.z);
    }

    void operator -=(const Vect3 &o) {
        x -= o.x;
        y -= o.y;
        z -= o.z;
    }

    Vect3 operator * (Scalar k) const {
        return Vect3(x*k, y*k, z*k);
    }

    void operator *=(Scalar k) {
        x *= k;          
        y *= k;
        z *= k;
    }

    bool operator == (const Vect3 &q) const {
        return (x==q.x) && (y==q.y) && (z==q.z);
    }

    void print_coordinates() const;
};

inline Vect3 operator * (Scalar k, const Vect3 &v){
    return v*k;
}

inline Vect3 mix (const Vect3 &a, const Vect3 &b, Scalar t){
    return (1-t)*a + t*b;
}

inline Vect3 nlerp (const Vect3 &a, const Vect3 &b, Scalar t){
    return mix(a, b, t).normalized();
}